﻿// The example is a modification of the example created by Phil Haack
// see http://haacked.com/archive/2009/04/14/using-jquery-grid-with-asp.net-mvc.aspx
using System.Linq;
using System.Web.Mvc;
using jqGridWeb.Models;
using System.Data.Objects;
using System.Web.Script.Serialization;
using System.Collections.Generic;
using System.Text;
using System;
using System.Data.Objects.SqlClient;
using System.Globalization;
using System.IO;

namespace jqGridWeb.Controllers
{
    // to send exceptions as json we define [HandleJsonException] attribute
    public class ExceptionInformation {
        public string Message { get; set; }
        public string Source { get; set; }
        public string StackTrace { get; set; }
    }

    public class HandleJsonExceptionAttribute : ActionFilterAttribute {
        // next class example are from the http://www.dotnetcurry.com/ShowArticle.aspx?ID=496
        public override void OnActionExecuted(ActionExecutedContext filterContext) {
            if (!filterContext.HttpContext.Request.IsAjaxRequest() || filterContext.Exception == null) return;
            filterContext.HttpContext.Response.StatusCode =
                (int)System.Net.HttpStatusCode.InternalServerError;

            var exInfo = new List<ExceptionInformation>();
            for (var ex = filterContext.Exception; ex != null; ex = ex.InnerException) {
                exInfo.Add(new ExceptionInformation {
                    Message = ex.Message,
                    Source = ex.Source,
                    StackTrace = ex.StackTrace
                });
            }
            filterContext.Result = new JsonResult {Data=exInfo};
            filterContext.ExceptionHandled = true;
        }
    }

    public class Filters {
        // ReSharper disable InconsistentNaming
        public enum GroupOp {
            AND,
            OR
        }
        public enum Operations {
            eq, // "equal"
            ne, // "not equal"
            lt, // "less"
            le, // "less or equal"
            gt, // "greater"
            ge, // "greater or equal"
            bw, // "begins with"
            bn, // "does not begin with"
            //in, // "in"
            //ni, // "not in"
            ew, // "ends with"
            en, // "does not end with"
            cn, // "contains"
            nc  // "does not contain"
        }
        public class Rule {
            public string field { get; set; }
            public Operations op { get; set; }
            public string data { get; set; }
        }

        public GroupOp groupOp { get; set; }
        public List<Rule> rules { get; set; }
        // ReSharper restore InconsistentNaming
        private static readonly string[] FormatMapping = {
            "(it.{0} = @p{1})",                 // "eq" - equal
            "(it.{0} <> @p{1})",                // "ne" - not equal
            "(it.{0} < @p{1})",                 // "lt" - less than
            "(it.{0} <= @p{1})",                // "le" - less than or equal to
            "(it.{0} > @p{1})",                 // "gt" - greater than
            "(it.{0} >= @p{1})",                // "ge" - greater than or equal to
            "(it.{0} LIKE (@p{1}+'%'))",        // "bw" - begins with
            "(it.{0} NOT LIKE (@p{1}+'%'))",    // "bn" - does not begin with
            "(it.{0} LIKE ('%'+@p{1}))",        // "ew" - ends with
            "(it.{0} NOT LIKE ('%'+@p{1}))",    // "en" - does not end with
            "(it.{0} LIKE ('%'+@p{1}+'%'))",    // "cn" - contains
            "(it.{0} NOT LIKE ('%'+@p{1}+'%'))" //" nc" - does not contain
        };
        internal ObjectQuery<T> FilterObjectSet<T> (ObjectQuery<T> inputQuery) where T : class {
            if (rules.Count <= 0)
                return inputQuery;

            var sb = new StringBuilder();
            var objParams = new List<ObjectParameter>(rules.Count);

            foreach (var rule in rules) {
                var propertyInfo = typeof (T).GetProperty (rule.field);
                if (propertyInfo == null)
                    continue; // skip wrong entries

                if (sb.Length != 0)
                    sb.Append(groupOp);

                var iParam = objParams.Count;
                sb.AppendFormat(FormatMapping[(int)rule.op], rule.field, iParam);

                ObjectParameter param;
                switch (propertyInfo.PropertyType.FullName) {
                    case "System.Int32":  // int
                        param = new ObjectParameter ("p" + iParam, Int32.Parse (rule.data));
                        break;
                    case "System.Int64":  // bigint
                        param = new ObjectParameter ("p" + iParam, Int64.Parse (rule.data));
                        break;
                    case "System.Int16":  // smallint
                        param = new ObjectParameter ("p" + iParam, Int16.Parse (rule.data));
                        break;
                    case "System.SByte":  // tinyint
                        param = new ObjectParameter ("p" + iParam, SByte.Parse (rule.data));
                        break;
                    case "System.Single": // Edm.Single, in SQL: float
                        param = new ObjectParameter ("p" + iParam, Single.Parse (rule.data));
                        break;
                    case "System.Double": // float(53), double precision
                        param = new ObjectParameter ("p" + iParam, Double.Parse (rule.data));
                        break;
                    case "System.Boolean": // Edm.Boolean, in SQL: bit
                        param = new ObjectParameter ("p" + iParam,
                            String.Compare(rule.data, "1", StringComparison.Ordinal) == 0 ||
                            String.Compare(rule.data, "yes", StringComparison.OrdinalIgnoreCase) == 0 ||
                            String.Compare(rule.data, "true", StringComparison.OrdinalIgnoreCase) == 0);
                        break;
                    default:
                        // TODO: Extend to other data types
                        // binary, date, datetimeoffset,
                        // decimal, numeric,
                        // money, smallmoney
                        // and so on

                        param = new ObjectParameter ("p" + iParam, rule.data);
                        break;
                }
                objParams.Add (param);
            }

            var filteredQuery = inputQuery.Where (sb.ToString());
            foreach (var objParam in objParams)
                filteredQuery.Parameters.Add (objParam);

            return filteredQuery;
        }
    }
    
    public class ExcelResult : ActionResult {
        private readonly DataForExcel _data;
        private readonly string _fileName;

        public ExcelResult (string[] headers, List<string[]> data, string fileName, string sheetName) {
            _data = new DataForExcel (headers, data, sheetName);
            _fileName = fileName;
        }

        public ExcelResult (string[] headers, DataForExcel.DataType[] colunmTypes, List<string[]> data, string fileName, string sheetName) {
            _data = new DataForExcel (headers, colunmTypes, data, sheetName);
            _fileName = fileName;
        }

        public override void ExecuteResult (ControllerContext context) {
            var response = context.HttpContext.Response;
            response.ClearContent();
            response.ClearHeaders();
            response.Cache.SetMaxAge (new TimeSpan (0));

            using (var stream = new MemoryStream()) {
                _data.CreateXlsxAndFillData (stream);

                //Return it to the client - strFile has been updated, so return it. 
                response.AddHeader ("content-disposition", "attachment; filename=" + _fileName);

                // see http://filext.com/faq/office_mime_types.php
                response.ContentType = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet";
                response.ContentEncoding = Encoding.UTF8;
                stream.WriteTo (response.OutputStream);
            }
            response.Flush();
            response.Close();
        }
    }

    //[HandleError]
    [HandleJsonException]  
    public class HomeController : Controller
    {
        public ActionResult Index()
        {
            ViewData["Message"] = "Welcome to ASP.NET MVC!";
            return View();
        }

        public JsonResult DynamicGridData(string sidx, string sord, int page, int rows, bool search, string filters)
        {
            Response.Cache.SetMaxAge (new TimeSpan (0));

            var context = new HaackOverflowEntities();
            var serializer = new JavaScriptSerializer();
            var f = (!search || string.IsNullOrEmpty (filters)) ? null : serializer.Deserialize<Filters> (filters);
            ObjectQuery<Question> filteredQuery = f == null ?
                context.Questions : f.FilterObjectSet (context.Questions);
            filteredQuery.MergeOption = MergeOption.NoTracking; // we don't want to update the data
            var totalRecords = filteredQuery.Count();

            var pagedQuery = filteredQuery.Skip ("it." + sidx + " " + sord, "@skip",
                                                new ObjectParameter ("skip", (page - 1) * rows))
                                         .Top ("@limit", new ObjectParameter ("limit", rows));
            // to be able to use ToString() below which is NOT exist in the LINQ to Entity
            // we should include in queryDetails only the properies which we will use below
            var queryDetails = (from item in pagedQuery
                                select new { item.Id, item.Votes, item.Title }).ToList();

            return Json(new {
                            total = (totalRecords + rows - 1) / rows,
                            page,
                            records = totalRecords,
                            rows = (from item in queryDetails
                                    select new[] {
                                        item.Id.ToString(CultureInfo.InvariantCulture),
                                        item.Votes.ToString(CultureInfo.InvariantCulture),
                                        item.Title
                                    }).ToList()
                        },
                        JsonRequestBehavior.AllowGet);
        }

// ReSharper disable InconsistentNaming
        public JsonResult ModifyGridData(string oper, string id, int? Votes, string Title) {
// ReSharper restore InconsistentNaming
            /*using (var context = new HaackOverflowEntities()) {
                if (String.Compare(oper, "add", StringComparison.Ordinal) == 0) {
                    var question = new Question {
                        Votes = Votes == null ? 0 : (int)Votes,
                        Title = Title,
                        Views = 1,
                        Slug = String.Empty,
                        Body = String.Empty,
                        DateCreated = DateTime.Now
                    };
                    context.AddToQuestions (question);
                    int changes = context.SaveChanges ();
                    if (changes > 0) {
                        return Json(new {question.Id, question.Votes});
                    }
                } else if (String.Compare(oper, "edit", StringComparison.Ordinal) == 0) {
                    int intId;
                    if (int.TryParse (id, out intId)) {
                        var question = context.Questions.SingleOrDefault (x => x.Id == intId);
                        if (question == null) {
                            throw new FileNotFoundException ("Item with the Id is not found");
                        }
                        if (Votes != null)
                            question.Votes = (int)Votes;
                        question.Title = Title;
                        context.SaveChanges ();
                        return Json (null);
                    } 
                    throw new ArgumentException ("The Id must be an integer", "id");
                }
            }*/
            throw new ArgumentException ("Wrong arguments");
            //oper="add", id="new_row"
        }

        public JsonResult GetIds (string term) {
            Response.Cache.SetMaxAge (new TimeSpan (0));

            var context = new HaackOverflowEntities();
            return Json ((from item in context.Questions
                          where SqlFunctions.StringConvert((decimal ?)item.Id).Contains(term) 
                          select new {
                              value = item.Id,
                              //votes = item.Votes,
                              title = item.Title
                          }).ToList (),
                         JsonRequestBehavior.AllowGet);
        }

        public JsonResult GetTitleAutocomplete (string term) {
            Response.Cache.SetMaxAge (new TimeSpan (0));

            var context = new HaackOverflowEntities();
            return Json ((from item in context.Questions
                          where item.Title.Contains (term)
                          select item.Title).ToList(),
                         JsonRequestBehavior.AllowGet);
        }

        private static readonly string[] HeadersQuestions = {
            "Id", "Votes", "Title"
        };
        private static readonly DataForExcel.DataType[] ColunmTypesQuestions = {
            DataForExcel.DataType.Integer,
            DataForExcel.DataType.Integer,
            DataForExcel.DataType.String
        };

        public ActionResult ExportAllQuestionsToExcel () {
            var context = new HaackOverflowEntities ();
            var questions = context.Questions;
            questions.MergeOption = MergeOption.NoTracking; // we don't want to update the data

            // to be able to use ToString() below which is NOT exist in the LINQ to Entity
            // we should include in query only the properies which we will use below
            var query = questions.ToList ();
            if (query.Count == 0)
                return new EmptyResult ();
            var data = new List<string[]> (query.Count);
            data.AddRange (query.Select (item => new[] {
                item.Id.ToString(CultureInfo.InvariantCulture),
                item.Votes.ToString(CultureInfo.InvariantCulture),
                item.Title
            }));

            return new ExcelResult (HeadersQuestions, ColunmTypesQuestions, data,
                                    "Questions.xlsx", "Questions");
        }
        
        public ActionResult About ()
        {
            return View();
        }
    }
}